@@ -12,6 +12,7 @@ use {
             path_from,
             PathAnchor,
         },
+        preview::PreviewTransformerConf,
         skin::SkinEntry,
         syntactic::SyntaxTheme,
         verb::ExecPattern,
@@ -39,6 +40,14 @@ macro_rules! overwrite_map {
     };
 }
 
+macro_rules! overwrite_vec {
+    ($dst: ident, $prop: ident, $src: ident) => {
+        for v in $src.$prop {
+            $dst.$prop.push(v);
+        }
+    };
+}
+
 /// The configuration read from conf.toml or conf.hjson file(s)
 #[derive(Default, Clone, Debug, Deserialize)]
 pub struct Conf {
@@ -86,6 +95,9 @@ pub struct Conf {
     #[serde(alias="kitty-graphics-transmission")]
     pub kitty_graphics_transmission: Option<TransmissionMedium>,
 
+    #[serde(default, alias="preview-transformers")]
+    pub preview_transformers: Vec<PreviewTransformerConf>,
+
     #[serde(alias="lines-after-match-in-preview")]
     pub lines_after_match_in_preview: Option<usize>,
 
@@ -221,10 +233,11 @@ impl Conf {
         overwrite!(self, lines_after_match_in_preview, conf);
         overwrite!(self, lines_before_match_in_preview, conf);
         self.verbs.append(&mut conf.verbs);
-        // the following maps are "additive": we can add entries from several
+        // the following prefs are "additive": we can add entries from several
         // config files and they still make sense
         overwrite_map!(self, special_paths, conf);
         overwrite_map!(self, ext_colors, conf);
+        overwrite_vec!(self, preview_transformers, conf);
         self.files.push(path);
         // read the imports
         for import in &conf.imports {
